const fs = require('fs')
class PdfTemplate {
    /**
 *
 * @param {Object} record
 * @param {Array<Object>} pages_data
 * @returns {Promise<Document>}
 */
    static fillup(record, pages_data) {
        console.log(`Start of mapping PDF... ${pages_data && pages_data[0] ? pages_data[0].code : ''}`);
        var document = {
            content: [],
            pageSize: pages_data && pages_data[0] && pages_data[0].pageSize ? pages_data[0].pageSize : 'A4',
            // pageMargins: [0, 30, 0, 0],
            pageMargins: [1, 31, 0, 0], // adjust margin for output pdf
            pageOrientation: pages_data && pages_data[0] && pages_data[0].pageOrientation ? pages_data[0].pageOrientation : 'portrait'
        };
        if (pages_data && pages_data.length) {
            var content = this.getContent(record, pages_data);
            if (content && content.length) document.content = content;
            var images = pages_data.map(v => v.background_image || null)
            if (images && images.length) {
                // var actions = [];
                var retrieved_images = [];
                images.forEach(img_url => {
                    if (img_url) retrieved_images.push(fs.readFileSync(img_url))
                })
                document.background = function (page) {
                    return [{
                        image: "form",
                        width: pages_data[0].pageWidth || 600
                    }]
                }
                if (!document.images) document.images = {}
                document.images.form = retrieved_images[0]
            }
        }
        console.log('End of mapping document :>> ', document);
        return document
    }

    /**
     * @param {Object} record 
     * @param {Array<Object>} pages 
     */
    static getContent(record, pages) {
        var content = [];
        pages.forEach((page, page_index) => {
            var widths = [], body = [], datas = [], rows = page.rowCount, cols = page.colCount;
            // initialize datas
            for (let a = 0; a < rows; a++) {
                datas[a] = []
                for (let b = 0; b < cols; b++) {
                    datas[a][b] = { text: " ", fontSize: 8, styles: 'cell', color: '#000', bold: true };
                }
            }
            if (page.datas && page.datas.length) {
                page.datas.forEach(config => {
                    var options = JSON.parse(JSON.stringify(config.options));
                    console.log('config.text :>> ', config.text);
                    options.text = config.text;
                    if (config.key && config.key.length && record) options.text = this.getValue(record, config.key);
                    if (config.qrcode) {
                        if (!options.qr) options.qr = options.text || 'This is an example'
                        delete options.text;
                    } else {
                        if (config.options.text_transform && options.text && typeof options.text === 'string') {
                            switch (config.options.text_transform) {
                                case 'U':
                                    options.text = options.text.toUpperCase();
                                    break;
                                case 'L':
                                    options.text = options.text.toLowerCase();
                                    break;
                                // case 'C':
                                //     options.text = options.text.toUpperCase();
                                //     break;
                            }
                        }
                        if (config.format && config.format.toUpperCase() !== 'NO FORMAT') {
                            options.text = this.applyFormat(config.datatype, options.text, config.format, config.condition)
                        }
                        console.log('options.text :>> ', options.text);
                        if (!options.text) options.text = " ";
                        if (!options.color) options.color = "#000";
                    }
                    if (datas[config.x] && datas[config.x][config.y])
                        datas[config.x][config.y] = options;
                })
            }

            // Setup widths
            for (let a = 0; a < cols; a++) widths.push(10);

            // setup body
            for (let a = 0; a < rows; a++) {
                var col_datas = [];
                for (let b = 0; b < cols; b++) col_datas.push(datas[a][b]);
                body.push(col_datas)
            }

            content.push({
                layout: "noBorders",
                table: {
                    widths,
                    body
                },
                // layout: {
                //     hLineWidth: function (i, node) {
                //         return 0.1;
                //     },
                //     vLineWidth: function (i, node) {
                //         return 0.1;
                //     },
                //     hLineColor: function (i, node) {
                //         return 'red';
                //     },
                //     vLineColor: function (i, node) {
                //         return 'red';
                //     },
                // },
                // styles: {
                //     cell: {
                //         color: '#FFF',
                //         opacity: `100%`
                //     }
                // }
            })

            if (pages.length > page_index + 1) {
                content.push(
                    {
                        layout: "noBorders",
                        table: {
                            widths: [10],
                            body: [
                                [

                                    {
                                        text: "",
                                        pageBreak: 'before'
                                    }

                                ]
                            ]
                        }
                    })
            }
        })

        return content;
    }

    /**
     * @param {Object} obj 
     * @param {String} keyword 
     */
    static getValue(obj, keyword) {
        var keywords = Array.isArray(keyword) ? keyword : keyword.split('.'), value = obj;
        if (obj && keywords && keywords.length) {
            keywords.forEach(element => {
                value = value[element] || "";
            });
            return value
        } return ""
    }


    static applyFormat(datatype, text, format, condition) {
        var formatted_text = text;
        switch (datatype) {
            case "Number":
                if (format === "AMOUNT") formatted_text = this.formatAmount(text);
                else if (format === "AMOUNT_NO_SIGN") formatted_text = this.formatAmount(text, true);
                break;
            case "Date":
                var type = null;
                switch (format) {
                    case "MMMM":
                        type = {
                            month: "long"
                        }
                        break;
                    case "YYYY":
                        type = {
                            year: "numeric"
                        }
                        break;
                    case "MMMM YYYY":
                        type = {
                            month: "long",
                            year: "numeric"
                        }
                        break;
                    case "MM/DD/YYYY":
                        type = {
                            year: "numeric",
                            month: "2-digit",
                            day: "2-digit"
                        }
                        break;
                    case "MMMM DD, YYYY":
                        type = {
                            year: "numeric",
                            month: "long",
                            day: "2-digit"
                        }
                        break;
                    case "MM/DD/YYYY TIME":
                        type = {
                            year: "numeric",
                            month: "2-digit",
                            day: "2-digit",
                            hour12: true,
                            hour: "2-digit",
                            minute: "2-digit"
                        }
                        break;
                    case "MMMM DD, YYYY TIME":
                        type = {
                            year: "numeric",
                            month: "long",
                            day: "2-digit",
                            hour12: true,
                            hour: "2-digit",
                            minute: "2-digit"
                        }
                        break;
                    case "MM DD, YYYY TIME":
                        type = "time"
                        break;
                }
                formatted_text = this.formatDate(text, type);
                break;
            case "Condition":
                formatted_text = text && condition && eval(this.replaceAll(condition, '$', text)) ? format : " ";
                break;
        }
        return formatted_text;
    }

    /**
    * @returns {String}
    * @param {Number} amount 
    */
    static formatAmount(amount, no_sign) {
        if (!amount || isNaN(amount)) return !no_sign ? "₱ 0.00" : "0.00";
        var parts = parseFloat(amount).toFixed(2).toString().split(".");
        parts[0] = parts[0].replace(/\B(?=(\d{3})+(?!\d))/g, ",");
        return `${!no_sign ? '₱ ' : ''}${parts.join(".")}`;
    }

    static formatDate(date, type) {
        try {
            var dt = new Date();
            if (date) {
                dt = new Date(date)
            } else return null
            if (!type) {
                type = {
                    year: "numeric",
                    month: "short",
                    day: "2-digit"
                }
            } else if (type === 'time') {
                type = {
                    hour12: true,
                    year: "numeric",
                    month: "short",
                    day: "2-digit",
                    hour: "2-digit",
                    minute: "2-digit"
                }
            }
            return dt.toLocaleString("en-US", type);
        } catch (error) {
            console.error(`Invalid Date: ${error}`);
            return "Invalid Date"
        }
    }

    static replaceAll(string, search, replace) {
        return string.split(search).join(replace);
    }
}

module.exports = PdfTemplate